@@ -465,7 +465,7 @@ class UserMethods:
 
         raise ValueError(
             'Could not find the input entity for {} ({}). Please read https://'
-            'docs.telethon.dev/en/latest/concepts/entities.html to'
+            'docs.telethon.dev/en/stable/concepts/entities.html to'
             ' find out more details.'
             .format(peer, type(peer).__name__)
         )
