@@ -11,4 +11,4 @@ mv /tmp/docs/* .
 git add constructors/ types/ methods/ index.html js/search.js css/ img/
 git commit --amend -m "Update documentation"
 git push --force
-git checkout master
+git checkout v1
